#include "CONGA.h"

/**
 * Wrapper function to implement Cluster Overlapping Newman Girvan Algorithm
 */
template <typename R>
void CONGA<R>::calculate()
{  
  //Breaks the network into connected components, and runs algorithm on each
  ForEachComponent<lui, R>(&(this->network), &CONGAComp, (void*)this);
}

/**
 * Cluster Overlapping Newman Girvan Algorithm implementation to run on each connected component
 * 
 * @param component Connected component of the network
 * @param instance Pointer to CONGA clustering algorithm instance to save communities to (function is static)
 * 
 * @return true
 */
template <typename R>
bool CONGA<R>::CONGAComp(map< lui, map< lui, R > >* component, void* instance)
{
  /*Print out the component*/
  ofstream fout("CurrComponent.txt");
  typename map < lui, map <lui, R> >::iterator it_gr;
  typename map <lui, R>::iterator it_ed;
  
  set < lui > nodes;
  
  for (it_gr = component->begin(); it_gr != component->end(); it_gr++){
    for (it_ed = it_gr->second.begin(); it_ed != it_gr->second.end(); it_ed++){
      fout << it_gr->first << ", " << it_ed->first << ", " << it_ed->second << endl;
      nodes.insert(it_gr->first);
      nodes.insert(it_ed->first);
    }
  }
   fout.close();
   
   if (nodes.size() < 3)
     return true;
   
   /*Run CIS to get # of clusters*/
   string CIS = "./cluster -alg=CIS -args=+-l 0.5 -d c+ -i=CurrComponent.txt -o=PreCIS.dat -d=,";   
   system(CIS.c_str());
   
   int clusterCount = lineCount("PreCIS.dat");
   
   system("rm PreCIS.dat");
   
   /*h controls how 'local' CONGO is (as opposed to CONGA which considers all shortest paths)*/
   string command = "java -Xmx2g -cp conga.jar CONGA CurrComponent.txt " + ((CONGA<R>*)(instance))->args;
   
  vector <string> aa;
  string extendArg = ("CONGAParam " + ((CONGA<double>*)(instance))->args);
  boost::split(aa, extendArg, boost::is_any_of(" "));
  
  char** av = new char*[aa.size()];
  for (sui r = 0; r < aa.size();r++){
    av[r] = (char*)(malloc((aa[r].size() + 1) * sizeof(char)));
    strcpy(av[r], aa[r].c_str());
  }
  map <string, vector < string > > subParam = getParameters(aa.size(), av);
  map <string, vector < string > >::iterator it_sub;
  if ((it_sub = subParam.find("n")) == subParam.end()){
    command += " -n " + boost::lexical_cast<string>(clusterCount);
  }
   
   system(command.c_str());
   
   /*Post Process and Save*/
  ifstream fin;
  vector <string> fields;
  openFile(&fin, "clusters-CurrComponent.txt");
  
  while (fline_tr(&fin, &fields, " ,")){
    if (fields.size() < 3) continue;
    
    set < lui > C;
    for (sui i = 0; i < fields.size(); i++){
	C.insert(boost::lexical_cast<lui>(fields[i]));
    }
    
    if (((CONGA<double>*)instance)->translate)
      ((CONGA<double>*)instance)->PrintTranslatedCommunity(&C);
    else
     ((CONGA<double>*)instance)->PrintCommunity(&C);
  }
  return true;
}

template bool CONGA<double>::CONGAComp(map< lui, map< lui, double > >* component, void* instance);
template bool CONGA<int>::CONGAComp(map< lui, map< lui, int > >* component, void* instance);
template void CONGA<double>::calculate();
template void CONGA<int>::calculate();