#ifndef CLIQPERC_H
#define CLIQPERC_H

#include "cluster.h"

/**
  * @file CliqPerc.h
  * @brief Implementation of Clique Percolation algorithm
  * 
  * @class CliqueCluster
  * @brief Clique Percolation Implementation
  * 
  * Calls the implemenation CFinder (CFinder_commandline), which must be installed in the same directory
  */
 template <typename R>
 class CliqueCluster: public ClusterAlg<R>
 {
  public:
    /**
     * Constructor
     * 
     * 	@param a Algorithm arguments
     */
    CliqueCluster(string a){
	this->args = "CliqueParams " + a;
        /*Get the clique size to percolate */
	vector <string> aa;
	boost::split(aa, this->args, boost::is_any_of(" "));
	
	char** av = new char*[aa.size()];
	for (sui r = 0; r < aa.size();r++){
	  av[r] = (char*)(malloc((aa[r].size() + 1) * sizeof(char)));
	  strcpy(av[r], aa[r].c_str());
	}
	map <string, vector < string > > subParam = getParameters(aa.size(), av);
	map <string, vector < string > >::iterator it_sub;
	
	if ((it_sub = subParam.find("k")) != subParam.end()){
	  ks = it_sub->second[0];
	} else {
	  cerr << "Invalid Clique Percolation Params (need size (-k))" << endl;
	  exit(2);
	}
	
	this->dens = new complex_density<bidirected_t>(0);
    };
    
    /**
     * Destructor
     * 
     * Empty
     */
    ~CliqueCluster(){};
    
    /**
     * Implementation
     */
    void calculate();
  private:
    string ks;
  };
#endif