#include "RRW.h"

/**
 * Wrapper function to implement repeated random walks.
 */
template <typename R>
void RRW<R>::calculate()
{  
  //Breaks the network into connected components, and runs algorithm on each
  ForEachComponent<lui, R>(&(this->network), &RRWComp, (void*)this);
}

/**
 * Repeated Random Walks implementation to run on each connected component
 * 
 * @param component Connected component of the network
 * @param instance Pointer to RRW clustering algorithm instance to save communities to (function is static)
 * @return true
 */
template <typename R>
bool RRW<R>::RRWComp(map <lui, map <lui, R> >* component, void* instance){
  /* Print out the component */
  ofstream fout("CurrComponent.txt");
  typename map < lui, map <lui, R> >::iterator it_gr;
  typename map <lui, R>::iterator it_ed;
  
  for (it_gr = component->begin(); it_gr != component->end(); it_gr++){
    for (it_ed = it_gr->second.begin(); it_ed != it_gr->second.end(); it_ed++){
      fout << it_gr->first << "\t" << it_ed->first << "\t" << it_ed->second << endl;
    }
  }
  
  /*Run RRW on the component*/
  string command = "java -Xmx2g RRW -i CurrComponent.txt -o RRWOutput.txt " + ((RRW<R>*)instance)->args;
  system(command.c_str());
  
  /*Post Process RRW output (undo tabs, get size and density, and concatenate all of them) and save the communities*/
  ifstream fin;
  if (!openFile(&fin, "RRWOutput.txt")){
    cerr << "Run Error - Invalid RRW parameters?" << endl;
    exit(3);
  }
  vector <string> fields;
  
  fout.close();
  
  while (fline_tr(&fin, &fields, " \t")){    
    set < lui > C;
    for (sui i = 0; i < fields.size(); i++){
	C.insert(boost::lexical_cast<lui>(fields[i]));
    }
    
    if (((RRW<double>*)instance)->translate)
      ((RRW<double>*)instance)->PrintTranslatedCommunity(&C);
    else
     ((RRW<double>*)instance)->PrintCommunity(&C);
  }
  
  return true;
}

template void RRW<double>::calculate();
template void RRW<int>::calculate();
template bool RRW<double>::RRWComp(map <lui, map < lui, double > >* component, void* instance);
template bool RRW<int>::RRWComp(map <lui, map < lui, int > >* component, void* instance);
