#include "SSDE.h"

/**
 * Wrapper function to implement Cluster Overlapping Newman Girvan Algorithm
 */
 template <typename R>
 void SSDE<R>::calculate()
 {
  //Breaks the network into connected components, and runs algorithm on each
  ForEachComponent<lui, R>(&(this->network), &SSDEComp, (void*)this);
 }

 /**
 * Sampled Spectral Distance Embedding implementation to run on each connected component
 * 
 * (Under Construction - Men at Work)
 * 
 * @param component Connected component of the network
 * @param instance Pointer to SSDE clustering algorithm instance to save communities to (function is static)
 * 
 * @return true
 */
template <typename R>
bool SSDE<R>::SSDEComp(map< lui, std::map< lui, R > >* component, void * instance)
{
  /* Print out the component */
  ofstream fout("CurrComponent.txt");
  typename map < lui, map <lui, R> >::iterator it_gr;
  typename map <lui, R>::iterator it_ed;
  
  map <lui, int> nodes;
  map <lui, int>::iterator it_n;
  
  for (it_gr = component->begin(); it_gr != component->end(); it_gr++){
    for (it_ed = it_gr->second.begin(); it_ed != it_gr->second.end(); it_ed++){
      fout << it_gr->first << " " << it_ed->first << " " << it_ed->second << endl;
      
      if ((it_n = nodes.find(it_gr->first)) != nodes.end()){
	it_n->second++;
      } else {
	nodes.insert(make_pair<lui, int>(it_gr->first, 1));
      }
      
      if ((it_n = nodes.find(it_ed->first)) != nodes.end()){
	it_n->second++;
      } else {
	nodes.insert(make_pair<lui, int>(it_ed->first, 1));
      }
    }
  }
   fout.close();
   
   fout.open("CurrNodes.txt");
   fout << nodes.size() << endl;
   
   for (it_n = nodes.begin(); it_n != nodes.end(); it_n++){
     fout << it_n->first << " " << it_n->second << endl;
   }
   
   if (nodes.size() < 3)
     return true;
   
   /*Run SSDE*/
   string SSDEcommand = "./SSDE CurrNodes.txt CurrComponent.txt run 1 0";
   if (nodes.size() < pow(2, 5))
   {
     SSDEcommand += " " + boost::lexical_cast<string>(max((int)log2(nodes.size()), 2));
   }
   system(SSDEcommand.c_str());
   
   /*Run GMM for soft clustering */
   //system("./cluster -alg=CIS -args=0 c -i=CurrComponent.txt -o=Count.txt");
   
   //int dCom = lineCount("Count.txt");
   system(("./sim 5 " + boost::lexical_cast<string>(3) + " 6 2 final_coords.txt CurrNodes.txt 0.001").c_str());
   
  // cerr << "sim " << dCom << " Communities from " << nodes.size() << "nodes" << endl;
   
   /*Run hard clustering post process */
   system(("./Cluster CurrNodes.txt CurrComponent.txt " + boost::lexical_cast<string>(3) + " 0 0").c_str());
      
   vector < set < lui > > stepCommunities;
   
   ifstream comin;
   openFile(&comin, "cluster_assign_randThreshold.txt");
   vector <string> fields;
   
   fline_tr(&comin, &fields, " ");
   while (fline_tr(&comin, &fields, " ")){
     if (fields.size() == 0)
       continue;
     
     lui thisNode = boost::lexical_cast<lui>(fields[0]);
     
     for (sui i = 1; i < fields.size(); i++){
       sui cluster = boost::lexical_cast<int>(fields[i]);
	while(stepCommunities.size() < (sui)(cluster + 1)){
	  set <lui> temp;
	  stepCommunities.push_back(temp);
	}
	
	stepCommunities[cluster].insert(thisNode);
     }
   }
   
   for (sui i = 0; i < stepCommunities.size(); i++){
     ((SSDE<double>*)instance)->PrintCommunity(&(stepCommunities[i]));
   }
   
   return true;
 }
 
 template void SSDE<double>::calculate();
 template void SSDE<int>::calculate();
 template bool SSDE<double>::SSDEComp(map< lui, std::map< lui, double > >* component, void * instance);
 template bool SSDE<int>::SSDEComp(map< lui, std::map< lui, int > >* component, void * instance);