#include "cluster.h"
#include "CliqPerc.h"
#include "CIS.h"
#include "RRW.h"
#include "CONGA.h"
#include "SLPA.h"
#include "OSLOM.h"
#include "COPRA.h"
#include "LCOMM.h"
#include "SSDE.h"

//Static variable definition
template <typename R>
density<bidirected_t>* ClusterAlg<R>::dens;

/** 
   * Factory function
   * 
   * @param type Algorithm to create class for
   * @param argList Arguments to use for algorithm
   * @param alpha Factor for positive weights
   * @param beta Factor for negative weights
   * @param weightThresh Minimum edge weight
   * @param translate Flag indicating whether or not to use original vertex names (from file)
   * 
   * @return Class set up to implement 'type' detection algorithm
    */
template <typename R>
ClusterAlg<R>* ClusterAlg<R>::Create(string type, string args, double alpha, double beta, double weightThresh, bool translate){
  //String compare the algorithm type, and simply create that algorithm's class
  if (type == "CIS"){
    ClusterAlg<R>* ret = new IterativeDensity<R>(args, alpha, beta, weightThresh, translate);
    return ret;
  } else 
    if (type == "RRW"){
    ClusterAlg<R>* ret = new RRW<R>(args);
    return ret;
  } else if (type == "3CL") {
    return ( new CliqueCluster<R>(args) );   
  } else if (type == "CNG"){
    ClusterAlg<R>* ret = new CONGA<R>(args);
    return ret;
  } else if (type == "SLP"){
    return new SLPA<R>(args);
  } else if (type == "OSL"){
    return new OSLOM<R>(args);
  } else if (type == "COP") {
   return new COPRA<R>(args);
  } else if (type == "COM") {
   return new LCOMM<R>(args);
  } if (type == "SDE") {
    return new SSDE<R>();
  } else {
    cerr << "Invalid algorithm type specified." << endl;
    exit(4);
  }
};

template ClusterAlg<int>* ClusterAlg<int>::Create(string type, string args, double alpha, double beta, double weightThresh, bool translate);
template ClusterAlg<double>* ClusterAlg<double>::Create(string type, string args, double alpha, double beta, double weightThresh, bool translate);
